/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.actionscript-facile.com
  Portions created by the Initial Developer are Copyright (C) 2010
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

*/
package
{
	import flash.net.URLRequest;
	import flash.display.Loader;
	import flash.geom.Point;
	import flash.display.Sprite;

	import flash.display.Bitmap;
	import flash.display.BitmapData;

	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.MouseEvent;

	import flash.geom.Matrix;


	/**
	 * Jeux Flash type Ticket à gratter.
	 * 
	 * @author Matthieu
	 * 
	 */
	public class TicketAgratter extends EventDispatcher
	{
		/* les éléments du ticket */
		private var oSpriteEndTicket:Sprite;
		private var oSpriteTicket:Sprite;
		private var oSpritePiece:Sprite;
		
		private var oContent:Sprite;
			
		private var sUrlImageGrattee:String;
		private var sUrlImageTicket:String;
		
		private var oPiece:Piece;
		
		private var oMatrix:Matrix;
		private var oMaskData:BitmapData;

		private var bStartGrattage:Boolean;
		private var nPourcentEndGrattage : int;
		private var oLoader : Loader;
		
		
		/**
		 * Constructeur.
		 * 
		 * @param oTargetContent : le DisplayObject sur lequel est à affiché le Ticket.
		 * @param nTxEndGrattage : le pourcentage à gratter pour considérer la fin du grattage du ticket.
		 * @param sUrlImageGratteeFin : l'url de l'image grattée / cachée.
		 * @param sUrlImageTicketAGratter : l'url de l'image à gratter.
		 * 
		 */
		public function TicketAgratter(oTargetContent:Sprite, nTxEndGrattage:int, sUrlImageGratteeFin:String, sUrlImageTicketAGratter:String = "") :void
		{
			this.oContent = oTargetContent;
			this.sUrlImageGrattee = sUrlImageGratteeFin;
			this.sUrlImageTicket = sUrlImageTicketAGratter;
			this.nPourcentEndGrattage = nTxEndGrattage;
			
			this.init();
		}


		/**
		 * Initialisation de tous les éléments.
		 * (en dehors du conctructeur pour optimiser le code.
		 * 
		 */
		private function init() : void
		{
			oPiece = new Piece();
			/* création des éléments graphiques */
			this.oSpriteTicket = new Sprite();
			this.oSpriteEndTicket = new Sprite();
			this.oSpritePiece = new Sprite();
		
			this.oContent.addChild(this.oSpriteTicket);
			this.oContent.addChild(this.oSpriteEndTicket);
			this.oContent.addChild(this.oSpritePiece);	
			
			this.bStartGrattage = false;// le grattage par l'utilisateur n'est pas commencé
			
			// chargement des images
			this.oLoader = new Loader();
			
			if (this.sUrlImageTicket != "")
			{
				this.oLoader.contentLoaderInfo.addEventListener(Event.COMPLETE, onLoadTicketAssets);
				this.oLoader.load(new URLRequest(sUrlImageTicket));
			}
			else
			{
				// chargement de la 2ème image
				this.oLoader.contentLoaderInfo.addEventListener(Event.COMPLETE, onTicketStart);
				this.oLoader.load(new URLRequest(sUrlImageGrattee));
			}
		}

		
		/**
		 * Détruit toutes les ressources utilisées par la classe.
		 * 
		 */
		public function destroy():void
		{
			oContent.removeChild(oSpriteTicket);
			oContent.removeChild(oSpriteEndTicket);
			oContent.removeChild(oSpritePiece);
			
			oMaskData.dispose();
			oMaskData = null;
			
			oLoader.unload();
			oLoader = null;
		}
		
		/**
		 * Chargement des images du ticket.
		 * 
		 */		
		private function onLoadTicketAssets(event:Event):void
		{
			var oPicture:Bitmap = Bitmap(event.target.content);
			this.oSpriteTicket.addChild(oPicture);
			this.oLoader.contentLoaderInfo.removeEventListener(Event.COMPLETE, onLoadTicketAssets);
			
			// chargement de la 2ème image
			this.oLoader.contentLoaderInfo.addEventListener(Event.COMPLETE, onTicketStart);
			this.oLoader.load(new URLRequest(sUrlImageGrattee));
		}
		
		
		/**
		 * Le Ticket est prêt à être gratté.
		 * 
		 */
		private function onTicketStart(event:Event):void
		{
			this.oLoader.contentLoaderInfo.removeEventListener(Event.COMPLETE, onTicketStart);
			
			var oPicture:Bitmap = Bitmap(event.target.content);

			this.oSpriteEndTicket.addChild(oPicture);
			
			// creation d'une représentation bitmap de la forme pour gratter
			var oPieceData:BitmapData = new BitmapData(this.oPiece.width, this.oPiece.height, true, 0);
			oPieceData.draw(this.oPiece);
			this.oMatrix = new Matrix();
						
			// création du masque de la Piece pour rendre visible l'image complete
			this.oMaskData = new BitmapData(oPicture.width, oPicture.height, true, 0);
			var oMask:Bitmap = new Bitmap(this.oMaskData);
			oMask.cacheAsBitmap = true;
			this.oSpritePiece.addChild(oMask);
			
			this.oSpriteEndTicket.cacheAsBitmap = true;
			this.oSpriteEndTicket.mask = oMask;
			
			// les évènements sur la souris
			this.oSpriteEndTicket.addEventListener(MouseEvent.MOUSE_DOWN, onGrattage);
			this.oSpriteEndTicket.addEventListener(MouseEvent.MOUSE_UP, onGrattage);
						
		}
		
		
		/**
		 * Grattage du ticket en cours par l'utilisateur.
		 * 
		 */
		private function onGrattage(event:MouseEvent):void
		{
			switch(event.type)
			{
				case MouseEvent.MOUSE_DOWN :
					this.oSpriteEndTicket.addEventListener(MouseEvent.MOUSE_MOVE, onGrattage);
				case MouseEvent.MOUSE_MOVE :
					updateTicket();
					break;
				case MouseEvent.MOUSE_UP :
					this.oSpriteEndTicket.removeEventListener(MouseEvent.MOUSE_MOVE, onGrattage);
					isFinishGratter();
			}
		}
		
		
		/**
		 * Gratte le ticket, en faisant apparaitre (par rapport à la piece)
		 * l'image grattée petit à petit.
		 * 
		 */
		private function updateTicket():void
		{
			this.oMatrix.tx = this.oSpriteTicket.mouseX - this.oPiece.width / 2;
			this.oMatrix.ty = this.oSpriteTicket.mouseY - this.oPiece.height / 2;
			
			Bitmap(this.oSpriteEndTicket.mask).bitmapData.draw(this.oPiece, this.oMatrix);
			
			if (this.bStartGrattage == false)
			{
				this.dispatchEvent(new Event(Event.OPEN));
				this.bStartGrattage = true;
			}		
		}
		
		
		/**
		 * Test si la zone est finie de gratter.
		 * 
		 */
		private function isFinishGratter():void
		{			
			// Teste les valeurs de pixels d'une image selon un seuil spécifié 
			// et définit les pixels qui réussissent le test sur de nouvelles valeurs de couleur
			var pixelsCaches:uint = oMaskData.threshold(oMaskData, oMaskData.rect, new Point(), "==", 0);
			var nPourcentGratte:Number = (1 - pixelsCaches / (oMaskData.width*oMaskData.height))*100;
			
			if (nPourcentGratte > this.nPourcentEndGrattage)
			{		
				this.oSpriteEndTicket.removeEventListener(MouseEvent.MOUSE_DOWN, onGrattage);
				this.oSpriteEndTicket.removeEventListener(MouseEvent.MOUSE_UP, onGrattage);		
				
				this.dispatchEvent(new Event(Event.COMPLETE));	
			}	

		}
		
	}
	
}


